#include <p32xxxx.h>
#include <plib.h>
#include "PIC32MX150pins.h"
#include "Nokia5110.h"

// DEVCFG3:
#pragma config IOL1WAY  = OFF       // Peripheral Pin Select Configuration, allow mult reconfig
#pragma config PMDL1WAY = OFF				// Peripheral Module Disable Config, allow mult reconfig
// DEVCFG2:
#pragma config FPLLODIV = DIV_2     // PLL Output Divider
#pragma config FPLLMUL  = MUL_20    // PLL Multiplier
#pragma config FPLLIDIV = DIV_2     // PLL Input Divider
// 8MHz / 2 * 20 / 2 = 40MHz
// DEVCFG1:
#pragma config FWDTEN   = OFF       // Watchdog Timer
#pragma config WDTPS    = PS1       // Watchdog Timer Postscale
#pragma config FCKSM    = CSDCMD    // Clock Switching & Fail Safe Clock Monitor
#pragma config FPBDIV   = DIV_1     // Peripheral Clock divisor
#pragma config OSCIOFNC = OFF       // CLKO Enable
#pragma config POSCMOD  = OFF       // Primary Oscillator
#pragma config IESO     = OFF       // Internal/External Switch-over
#pragma config FSOSCEN  = OFF       // Secondary Oscillator Enable (KLO was off)
#pragma config FNOSC    = FRCPLL    // Oscillator Selection
// DEVCFG0:
#pragma config CP       = OFF       // Code Protect
#pragma config BWP      = ON        // Boot Flash Write Protect
#pragma config PWP      = OFF       // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx1  // ICE/ICD Comm Channel Select
#pragma config JTAGEN   = OFF       // JTAG Enable
#pragma config DEBUG    = OFF       // Background Debugger Enable

#define SYS_FREQ 40000000L

void init_analog(void);
int read_analog(int pin);

int main(void) {
  CFGCONbits.JTAGEN = 0; // turn off JTAG, get back those pins for IO use

  SYSTEMConfigPerformance(SYS_FREQ);
  INTEnableSystemMultiVectoredInt();

  // set the pins to functions
  PIC32MX150_setup_pins();

  /////////////////////////////////
  // H bridge setup
  /////////////////////////////////
  // A1 is OC2, B3 is OC1
  // B2 is direction for OC2, B4 is direction for OC1
  // Timer2 for frequency
  LATBbits.LATB2 = 0;
  LATBbits.LATB4 = 0;
  TRISBbits.TRISB2 = 0;
  TRISBbits.TRISB4 = 0;

  // set up OC1 and OC2 to 20000Hz
  T2CON = 0; // turn off TMR2
  OC1CON = 0; //turn off OC1
  OC2CON = 0; //turn off OC2
  T2CONbits.TCKPS = 2; // set prescaler 1:4
  TMR2 = 0; // start TMR2 counting from zero.
  PR2 = 499; // set period match value
  /* so 20000Hz from 499 from (40MHz / 4 prescaler / 20000Hz) - 1 = 499 */
  OC1CONbits.OCM = 0b110; // enable PWM mode with no fault protection
  OC2CONbits.OCM = 0b110; // enable PWM mode with no fault protection
  OC1RS = 0;
  OC1R = 0;
  OC2RS = 0;
  OC2R = 0;
  T2CONbits.ON = 1; // turn on TMR2
  OC1CONbits.ON = 1; // turn OC1 back on
  OC2CONbits.ON = 1; // turn OC2 back on

  /////////////////////////////////
  // LCD setup
  /////////////////////////////////
  // A4 is SDO2
  // B15 is SCK2
  // B5 is DC
  // B10 is Reset
  // B7 is SCE
  LcdInitialize();
  LcdClear();

  /////////////////////////////////
  // IR distance sensor setup
  /////////////////////////////////
  // B9 is SDA1
  // B8 is SCL1


  /////////////////////////////////
  // 8 other pins setup
  /////////////////////////////////
  // B6
  // B11
  // B12
  // B13
  // B14
  // A0
  // A2
  // A3

  // spin for 2 seconds
  WriteCoreTimer(0);
  OC1RS = 499;
  while (ReadCoreTimer() < 40000000) {
  }
  OC1RS = 0;

  // walk for 2 seconds
  WriteCoreTimer(0);
  OC2RS = 999;
  while (ReadCoreTimer() < 40000000) {
  }
  OC2RS = 0;

  // walk and turn
  OC1RS = 200;
  OC2RS = 499;

  char LCDmessage[11];
  int i = 0;
  while (1) {
    // start timer so that the loop takes 1 second
    WriteCoreTimer(0);

    // write to the LCD
    sprintf(LCDmessage, "i = %d", i);
    gotoXY(0, 0);
    LcdString(LCDmessage);
    i++;

    // wait the rest of the 1 second
    while (ReadCoreTimer() < 20000000) {
    }
  }
  return 0;
}

void init_analog(void) {
  // Set all AN pins (B0-B15) for analog input
  TRISBSET = 0xFFFF;
  //AD1PCFGCLR = 0xFFFF;

  AD1CON1bits.FORM = 0b000; // Select 16-bit integer output
  AD1CON1bits.ASAM = 1; // Auto Sample
  AD1CON1bits.SSRC = 0b111; // Auto convert sampled data to digital
  AD1CON2bits.VCFG = 0b000; // Select the vrefs to be Vdd and Vss

  AD1CON2bits.CSCNA = 1; // Enable ADC Scaning
  AD1CSSL = 0xFFFF; // Select all AN pins for scanning
  AD1CON2bits.SMPI = 16 - 1; // Select 3 samples per interupt
  /* In scanning mode, this number must be at least as big as
   * the number of channels you are scanning/
   */

  AD1CON3bits.ADRC = 0; // Use PBCLK as clock source
  // Tpb = 1/80MHz = 12.5ns
  // Sample at 100ksps to reduce noise.
  // For ADC minimum, use 2 Tad, so Tad = 10us / (12+2) = 714ns
  // Sample time is Tad*2, so Tsamp = 1.428us
  // ADC clock divisor = 714ns / (1/80MHz) = 57
  // Actual sample rate is 1/(57*(12+2)*12.5ns) = 100ksps
  // So ADCS is 57 and SAMC is 2
  AD1CON3bits.SAMC = 2;
  AD1CON3bits.ADCS = 57;
  AD1CON1bits.ADON = 1; // Turn the ADC on
}

int read_analog(int pin) {
  switch (pin) {
    case 0:
      return ADC1BUF0;
      break;
    case 1:
      return ADC1BUF1;
      break;
    case 2:
      return ADC1BUF2;
      break;
    case 3:
      return ADC1BUF3;
      break;
    case 4:
      return ADC1BUF4;
      break;
    case 5:
      return ADC1BUF5;
      break;
    case 6:
      return ADC1BUF6;
      break;
  }
}

